/*
    Copyright (c) 2002, 2003, bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/

#import "BDNotQualifier.h"


@implementation BDNotQualifier

- (id)initWithQualifier:(BDQualifier *)qualifier
{
    if (self = [super init]) {
        _qualifier = [qualifier retain];
    }

    return self;
}

- (void)dealloc
{
    [_qualifier release];
    [super dealloc];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"not (%@)", [_qualifier description]];
}

- (BOOL)evaluateWithObject:(id)object
{
    BOOL result = NO;

    if (_qualifier != nil) {
        result = ![_qualifier evaluateWithObject:object];
    }

    return result;
}

- (unsigned)count
{
    int count = 1;
    
    if (_qualifier != nil) {
        count += [_qualifier count];
    }

    return count;
}

- (NSArray *)subqualifiers
{
    NSArray *subqualifiers;

    if (_qualifier != nil) {
        subqualifiers = [NSArray arrayWithObject:_qualifier];
    } else {
        subqualifiers = [NSArray array];
    }

    return subqualifiers;
}

- (void)addQualifierKeysToSet:(NSMutableSet *)qualKeys
{
    // Add the sub-qualifier's keys to the set.

    [_qualifier addQualifierKeysToSet:qualKeys];
}

/* Accessing a qualifier's binding keys */

- (NSArray *)bindingKeys
{
    return [_qualifier bindingKeys];
}

- (NSString *)keyPathForBindingKey:(NSString *)key
{
    return [_qualifier keyPathForBindingKey:key];
}

- (BDQualifier *)qualifierWithBindings:(NSDictionary *)bindings
                  requiresAllVariables:(BOOL)requiresAll
{
    BDQualifier *result;
    BDQualifier *newQualifier;

    newQualifier = [_qualifier qualifierWithBindings:bindings
                                requiresAllVariables:requiresAll];

    if (newQualifier != nil) {
        result = [[[BDNotQualifier alloc] initWithQualifier:newQualifier] autorelease];
    } else {
        result = nil;
    }

    return result;
}

/* NSCoding Protocol */

- (id)initWithCoder:(NSCoder *)coder
{
    BDQualifier *qualifier;
    
    qualifier = [coder decodeObject];
    
    return [self initWithQualifier:qualifier];
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeObject:_qualifier];
}

@end
