/*
    Copyright (c) 2002, 2003, bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/

%{
#include <string.h>
#include "BDQualifierParser.h"
#include "BDQualifierNode.h"
#include "BDStringNode.h"

extern int BDQualifierParser_YYINPUT(char *buf, int max_size);

void BDQualifierParser_reset_state(void);
    
#undef YY_INPUT
#define YY_INPUT(b,r,s) (r = BDQualifierParser_YYINPUT(b,s))
%}

PM	[+-]
DI	[0-9]
EX	[Ee]{PM}?{DI}+
ID	[A-Za-z_][A-Za-z0-9_]*
KEYPATH	{ID}(\.{ID})*
WSP	[ \t\r\n]
CONTAINS	[Cc][Oo][Nn][Tt][Aa][Ii][Nn][Ss]
LIKE	[Ll][Ii][Kk][Ee]
CASEINSENSITIVELIKE	[Cc][Aa][Ss][Ee][Ii][Nn][Ss][Ee][Nn][Ss][Ii][Tt][Ii][Vv][Ee][Ll][Ii][Kk][Ee]
TRUE	[Tt][Rr][Uu][Ee]
FALSE	[Ff][Aa][Ll][Ss][Ee]
NIL	[Nn][Ii][Ll]
NULL	[Nn][Uu][Ll][Ll]
AND	[Aa][Nn][Dd]
OR	[Oo][Rr]
NOT	[Nn][Oo][Tt]

%%

{CONTAINS}		{	/* CONTAINS */
				return CONTAINS;
			}

{LIKE}			{	/* LIKE */
				return LIKE;
			}

{CASEINSENSITIVELIKE}	{	/* CASE_INSENSITIVE_LIKE */
				return CASE_INSENSITIVE_LIKE;
			}

\*{TRUE}\*		{	/* QUALIFIER_TRUE */
				return QUALIFIER_TRUE;
                        }

\*{FALSE}\*		{	/* QUALIFIER_FALSE */
				return QUALIFIER_FALSE;
                        }

{PM}?{DI}*("."{DI}*)?{EX}?	{	/* NUMBER */
				BDQualifierParser_lval.doubleValue = atof(BDQualifierParser_text);
				return NUMBER;
			}

\'(\\.|[^\\'])*\'	{	/* STRING */
				BDQualifierParser_lval.stringValue = BDStringNodeCreate(&(BDQualifierParser_text[1]));
				BDQualifierParser_lval.stringValue->string[BDQualifierParser_leng - 2] = '\0';
				return STRING;
			}

{NIL}			{	/* NIL */
				return NIL;
			}

{NULL}			{	/* NIL */
				return NIL;
			}

{TRUE}			{	/* TRUE */
				return TRUE;
			}

{FALSE}			{	/* FALSE */
				return FALSE;
			}

{AND}			{	/* AND */
				return AND;
			}

{OR}			{	/* OR */
				return OR;
			}

{NOT}			{	/* NOT */
				return NOT;
			}

{ID}			{	/* IDENTIFIER */
				BDQualifierParser_lval.stringValue = BDStringNodeCreate(BDQualifierParser_text);
				return IDENTIFIER;
			}

{KEYPATH}		{	/* KEYPATH */
				BDQualifierParser_lval.stringValue = BDStringNodeCreate(BDQualifierParser_text);
				return KEYPATH;
			}

\${ID}			{	/* VARIABLE */
				BDQualifierParser_lval.stringValue = BDStringNodeCreate(&(BDQualifierParser_text[1]));
				return VARIABLE;
			}

\(			{	/* LEFT_PAREN */
				return LEFT_PAREN;
			}

\)			{	/* RIGHT_PAREN */
				return RIGHT_PAREN;
			}

=			{	/* EQUAL_TO */
				return EQUAL_TO;
			}

==			{	/* EQUAL_TO_2 */
				return EQUAL_TO_2;
			}

!=			{	/* NOT_EQUAL_TO */
				return NOT_EQUAL_TO;
			}

\<			{	/* LESS_THAN */
				return LESS_THAN;
			}

\>			{	/* GREATER_THAN */
				return GREATER_THAN;
			}

\<=			{	/* LESS_THAN_OR_EQUAL_TO */
				return LESS_THAN_OR_EQUAL_TO;
			}

\>=			{	/* GREATER_THAN_OR_EQUAL_TO */
				return GREATER_THAN_OR_EQUAL_TO;
			}

{WSP}+			{	/* skip whitespace */
			}

%%

void BDQualifierParser_error(char *s)
{
    extern BDQualifierNode *BDQualifierParser_lastParsedNode;

    if (BDQualifierParser_lastParsedNode != NULL) {
        BDQualifierNodeRelease(BDQualifierParser_lastParsedNode);
        BDQualifierParser_lastParsedNode = NULL;
    }
}

int BDQualifierParser_wrap()
{
    return -1;
}

void BDQualifierParser_reset_state()
{
    BEGIN(INITIAL);
}
