/*
    Copyright (c) 2002, 2003, bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include "BDQualifierNode.h"


void BDQualifierNodeFree(BDQualifierNode *node);
void BDQualifierNodeFreeAndNode(BDQualifierNode *node);
void BDQualifierNodeFreeOrNode(BDQualifierNode *node);
void BDQualifierNodeFreeNotNode(BDQualifierNode *node);
void BDQualifierNodeFreeKeyValueNode(BDQualifierNode *node);
void BDQualifierNodeFreeKeyComparisonNode(BDQualifierNode *node);
void BDQualifierNodeFreeBooleanNode(BDQualifierNode *node);


BDQualifierNode *BDQualifierNodeCreateAndNode(BDQualifierNode *left,
                                              BDQualifierNode *right)
{
    BDQualifierNode *node = NULL;
    node = malloc(sizeof(BDQualifierNode));
    if (node != NULL) {
        node->rc = 1;
        node->type = kBDAndQualifierNodeType;
        node->data.andNode.leftQual = BDQualifierNodeRetain(left);
        node->data.andNode.rightQual = BDQualifierNodeRetain(right);
    }
    return node;
}

BDQualifierNode *BDQualifierNodeCreateOrNode(BDQualifierNode *left,
                                             BDQualifierNode *right)
{
    BDQualifierNode *node = NULL;
    node = malloc(sizeof(BDQualifierNode));
    if (node != NULL) {
        node->rc = 1;
        node->type = kBDOrQualifierNodeType;
        node->data.orNode.leftQual = BDQualifierNodeRetain(left);
        node->data.orNode.rightQual = BDQualifierNodeRetain(right);
    }
    return node;
}

BDQualifierNode *BDQualifierNodeCreateNotNode(BDQualifierNode *qual)
{
    BDQualifierNode *node = NULL;
    node = malloc(sizeof(BDQualifierNode));
    if (node != NULL) {
        node->rc = 1;
        node->type = kBDNotQualifierNodeType;
        node->data.notNode.qual = BDQualifierNodeRetain(qual);
    }
    return node;
}

BDQualifierNode *BDQualifierNodeCreateKeyValueNode(BDStringNode *key,
                                                   BDStringNode *operatorSelector,
                                                   enum BDQualifierNodeValueType valueType,
                                                   double doubleValue,
                                                   BDStringNode *stringValue,
                                                   BDStringNode *typedStringType,
                                                   BDStringNode *typedStringValue,
                                                   BDStringNode *variableValue,
                                                   int booleanValue)
{
    BDQualifierNode *node = NULL;
    
    node = malloc(sizeof(BDQualifierNode));

    if (node != NULL) {
        node->rc = 1;

        node->type = kBDKeyValueQualifierNodeType;

        node->data.keyValueNode.key = BDStringNodeCopyString(key);
        node->data.keyValueNode.operatorSelector = BDStringNodeCopyString(operatorSelector);
        node->data.keyValueNode.valueType = valueType;

        switch (valueType) {
            case kBDQualifierNodeValueNumber:
                node->data.keyValueNode.valueData.doubleValue = doubleValue;
                break;
                
            case kBDQualifierNodeValueString:
                node->data.keyValueNode.valueData.stringValue = BDStringNodeCopyString(stringValue);
                break;
                
            case kBDQualifierNodeValueNil:
                /* Do nothing */
                break;
                
            case kBDQualifierNodeValueTypedString:
                node->data.keyValueNode.valueData.typedStringType = BDStringNodeCopyString(typedStringType);
                node->data.keyValueNode.valueData.typedStringValue = BDStringNodeCopyString(typedStringValue);
                break;
                
            case kBDQualifierNodeValueVariable:
                node->data.keyValueNode.valueData.variableValue = BDStringNodeCopyString(variableValue);
                break;
                
            case kBDQualifierNodeValueBoolean:
                node->data.keyValueNode.valueData.booleanValue = booleanValue;
                break;
                
            default:
                /* Do nothing */
                break;
        }
    }
    return node;
}

BDQualifierNode *BDQualifierNodeCreateKeyComparisonNode(BDStringNode *leftKey,
                                                        BDStringNode *operatorSelector,
                                                        BDStringNode *rightKey)
{
    BDQualifierNode *node = NULL;

    node = malloc(sizeof(BDQualifierNode));
    
    if (node != NULL) {
        node->rc = 1;
        node->type = kBDKeyComparisonQualifierNodeType;
        node->data.keyComparisonNode.leftKey = BDStringNodeCopyString(leftKey);
        node->data.keyComparisonNode.operatorSelector = BDStringNodeCopyString(operatorSelector);
        node->data.keyComparisonNode.rightKey = BDStringNodeCopyString(rightKey);
    }

    return node;
}

BDQualifierNode *BDQualifierNodeCreateBooleanNode(int value)
{
    BDQualifierNode *node = NULL;

    node = malloc(sizeof(BDQualifierNode));

    if (node != NULL) {
        node->rc = 1;
        node->type = kBDBooleanQualifierNodeType;
        node->data.booleanNode = value;
    }

    return node;
}

BDQualifierNode *BDQualifierNodeRetain(BDQualifierNode *node)
{
    if (node != NULL) {
        node->rc++;
    }

    return node;
}


void BDQualifierNodeRelease(BDQualifierNode *node)
{
    if (node != NULL) {
        node->rc--;
        if (node->rc == 0) {
            BDQualifierNodeFree(node);
        }
    }
}


void BDQualifierNodeFree(BDQualifierNode *node)
{
    if (node != NULL) {
        switch (node->type) {
            case kBDAndQualifierNodeType:
                BDQualifierNodeFreeAndNode(node);
                break;

            case kBDOrQualifierNodeType:
                BDQualifierNodeFreeOrNode(node);
                break;

            case kBDNotQualifierNodeType:
                BDQualifierNodeFreeNotNode(node);
                break;

            case kBDKeyValueQualifierNodeType:
                BDQualifierNodeFreeKeyValueNode(node);
                break;

            case kBDKeyComparisonQualifierNodeType:
                BDQualifierNodeFreeKeyComparisonNode(node);
                break;

            case kBDBooleanQualifierNodeType:
                BDQualifierNodeFreeBooleanNode(node);
                break;

            default:
                // Do nothing
                break;
        }

        node->type = kBDInvalidQualifierNodeType;

        free(node);
    }
}


void BDQualifierNodeFreeAndNode(BDQualifierNode *node)
{
    BDQualifierNodeRelease(node->data.andNode.leftQual);
    node->data.andNode.leftQual = NULL;

    BDQualifierNodeRelease(node->data.andNode.rightQual);
    node->data.andNode.rightQual = NULL;
}


void BDQualifierNodeFreeOrNode(BDQualifierNode *node)
{
    BDQualifierNodeRelease(node->data.orNode.leftQual);
    node->data.orNode.leftQual = NULL;

    BDQualifierNodeRelease(node->data.orNode.rightQual);
    node->data.orNode.rightQual = NULL;
}


void BDQualifierNodeFreeNotNode(BDQualifierNode *node)
{
    BDQualifierNodeRelease(node->data.notNode.qual);
    node->data.notNode.qual = NULL;
}


void BDQualifierNodeFreeKeyValueNode(BDQualifierNode *node)
{
    if (node->data.keyValueNode.key != NULL) {
        free(node->data.keyValueNode.key);
        node->data.keyValueNode.key = NULL;
    }

    if (node->data.keyValueNode.operatorSelector != NULL) {
        free(node->data.keyValueNode.operatorSelector);
        node->data.keyValueNode.operatorSelector = NULL;
    }

    switch (node->data.keyValueNode.valueType) {
        case kBDQualifierNodeValueNumber:
            // Do nothing
            break;

        case kBDQualifierNodeValueString:
            if (node->data.keyValueNode.valueData.stringValue != NULL) {
                free(node->data.keyValueNode.valueData.stringValue);
                node->data.keyValueNode.valueData.stringValue = NULL;
            }
            break;

        case kBDQualifierNodeValueNil:
            // Do nothing
            break;

        case kBDQualifierNodeValueTypedString:
            if (node->data.keyValueNode.valueData.typedStringType != NULL) {
                free(node->data.keyValueNode.valueData.typedStringType);
                node->data.keyValueNode.valueData.typedStringType = NULL;
            }
            if (node->data.keyValueNode.valueData.typedStringValue != NULL) {
                free(node->data.keyValueNode.valueData.typedStringValue);
                node->data.keyValueNode.valueData.typedStringValue = NULL;
            }
            break;

        case kBDQualifierNodeValueVariable:
            if (node->data.keyValueNode.valueData.variableValue != NULL) {
                free(node->data.keyValueNode.valueData.variableValue);
                node->data.keyValueNode.valueData.variableValue = NULL;
            }
            break;

        case kBDQualifierNodeValueBoolean:
            // Do nothing
            break;

        default:
            // Do nothing
            break;
    }
}


void BDQualifierNodeFreeKeyComparisonNode(BDQualifierNode *node)
{
    if (node->data.keyComparisonNode.leftKey != NULL) {
        free(node->data.keyComparisonNode.leftKey);
        node->data.keyComparisonNode.leftKey = NULL;
    }

    if (node->data.keyComparisonNode.operatorSelector != NULL) {
        free(node->data.keyComparisonNode.operatorSelector);
        node->data.keyComparisonNode.operatorSelector = NULL;
    }

    if (node->data.keyComparisonNode.rightKey != NULL) {
        free(node->data.keyComparisonNode.rightKey);
        node->data.keyComparisonNode.rightKey = NULL;
    }
}


void BDQualifierNodeFreeBooleanNode(BDQualifierNode *node)
{
    // Do nothing
}


void BDQualifierNodeDump(BDQualifierNode *node, FILE *file)
{
    if (node != NULL) {
        switch (node->type) {
            case kBDAndQualifierNodeType:
                fprintf(file, "(BDAndQualifier<%d> ", node->rc);
                BDQualifierNodeDump(node->data.andNode.leftQual, file);
                fprintf(file, " ");
                BDQualifierNodeDump(node->data.andNode.rightQual, file);
                fprintf(file, ")");
                break;

            case kBDOrQualifierNodeType:
                fprintf(file, "(BDOrQualifier<%d> ", node->rc);
                BDQualifierNodeDump(node->data.orNode.leftQual, file);
                fprintf(file, " ");
                BDQualifierNodeDump(node->data.orNode.rightQual, file);
                fprintf(file, ")");
                break;

            case kBDNotQualifierNodeType:
                fprintf(file, "(BDNotQualifier<%d> ", node->rc);
                BDQualifierNodeDump(node->data.notNode.qual, file);
                fprintf(file, ")");
                break;

            case kBDKeyValueQualifierNodeType:
                fprintf(file, "(BDKeyValueQualifier<%d> %s %s ",
                        node->rc,
                        node->data.keyValueNode.key,
                        node->data.keyValueNode.operatorSelector);

                switch (node->data.keyValueNode.valueType) {
                    case kBDQualifierNodeValueNumber:
                        fprintf(file, "%f", node->data.keyValueNode.valueData.doubleValue);
                        break;

                    case kBDQualifierNodeValueString:
                        fprintf(file, "'%s'", node->data.keyValueNode.valueData.stringValue);
                        break;

                    case kBDQualifierNodeValueNil:
                        fprintf(file, "nil");
                        break;

                    case kBDQualifierNodeValueTypedString:
                        fprintf(file, "(%s) '%s'",
                                node->data.keyValueNode.valueData.typedStringType,
                                node->data.keyValueNode.valueData.typedStringValue);
                        break;

                    case kBDQualifierNodeValueVariable:
                        fprintf(file, "$%s", node->data.keyValueNode.valueData.variableValue);
                        break;

                    case kBDQualifierNodeValueBoolean:
                        if (node->data.keyValueNode.valueData.booleanValue) {
                            fprintf(file, "true");
                        } else {
                            fprintf(file, "false");
                        }
                        break;

                    default:
                        /* Do nothing */
                        break;
                }
                    fprintf(file, ")");
                break;

            case kBDKeyComparisonQualifierNodeType:
                fprintf(file, "(BDKeyComparisonQualifier<%d> %s %s %s)",
                        node->rc,
                        node->data.keyComparisonNode.leftKey,
                        node->data.keyComparisonNode.operatorSelector,
                        node->data.keyComparisonNode.rightKey);
                break;

            case kBDBooleanQualifierNodeType:
                fprintf(file, "(BDBooleanQualifier<%d> %d)",
                        node->rc,
                        node->data.booleanNode);
                break;

            default:
                fprintf(file, "(UNKNOWN<%d>)", node->rc);
                break;
        }
    } else {
        fprintf(file, "ERROR: NULL");
    }
}
