/*
    Copyright (c) 2002, 2003, bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/

%{

#include <string.h>
#include <stdlib.h>

#include "BDQualifierNode.h"
#include "BDStringNode.h"

BDQualifierNode *BDQualifierParser_lastParsedNode;

extern void BDQualifierParser_error(char *s);
extern int BDQualifierParser_lex();

%}

%union {
    double doubleValue;
    struct BDStringNode *stringValue;
    struct BDQualifierNode *qualifierValue;
}

%token <doubleValue> NUMBER
%token <stringValue> STRING
%token NIL
%token AND OR NOT

%token EQUAL_TO EQUAL_TO_2
%token NOT_EQUAL_TO
%token LESS_THAN GREATER_THAN
%token LESS_THAN_OR_EQUAL_TO GREATER_THAN_OR_EQUAL_TO
%token CONTAINS LIKE CASE_INSENSITIVE_LIKE
%token LEFT_PAREN RIGHT_PAREN

%token <stringValue> IDENTIFIER
%token <stringValue> KEYPATH
%token <stringValue> VARIABLE
%token TRUE
%token FALSE
%token QUALIFIER_TRUE
%token QUALIFIER_FALSE

%left SELECTOR
%left AND OR
%right NOT

%type <qualifierValue> qualifier
%type <stringValue> key
%type <stringValue> selector

%%

qualifier : LEFT_PAREN qualifier RIGHT_PAREN    /* nested qualifier */
                {
                    $$ = $2;
                    
                    BDQualifierParser_lastParsedNode = $$;
                }

	  /* BDBooleanQualifier */
          | QUALIFIER_TRUE
		{
                    $$ = BDQualifierNodeCreateBooleanNode(1);
    
                    BDQualifierParser_lastParsedNode = $$;
                }
	  | QUALIFIER_FALSE
                {
		    $$ = BDQualifierNodeCreateBooleanNode(0);

		    BDQualifierParser_lastParsedNode = $$;
		}

	  /* BDKeyValueQualifier */
          | key selector NUMBER
                {
                    $$ = BDQualifierNodeCreateKeyValueNode($1, $2,
                                                           kBDQualifierNodeValueNumber,
                                                           $3, NULL, NULL, NULL, NULL,
                                                           0);
                    
                    BDQualifierParser_lastParsedNode = $$;

                    BDStringNodeRelease($1);
                    BDStringNodeRelease($2);
                }

          | key selector STRING
                {
                    $$ = BDQualifierNodeCreateKeyValueNode($1, $2,
                                                           kBDQualifierNodeValueString,
                                                           0, $3, NULL, NULL, NULL,
                                                           0);
                    
                    BDQualifierParser_lastParsedNode = $$;

                    BDStringNodeRelease($1);
                    BDStringNodeRelease($2);
                    BDStringNodeRelease($3);
                }

          | key selector NIL
                {
                    $$ = BDQualifierNodeCreateKeyValueNode($1, $2,
                                                           kBDQualifierNodeValueNil,
                                                           0, NULL, NULL, NULL, NULL,
                                                           0);
                    
                    BDQualifierParser_lastParsedNode = $$;

                    BDStringNodeRelease($1);
                    BDStringNodeRelease($2);
                }

          | key selector LEFT_PAREN IDENTIFIER RIGHT_PAREN STRING
                {
                    $$ = BDQualifierNodeCreateKeyValueNode($1, $2,
                                                           kBDQualifierNodeValueTypedString,
                                                           0, NULL, $4, $6, NULL,
                                                           0);
                    
                    BDQualifierParser_lastParsedNode = $$;

                    BDStringNodeRelease($1);
                    BDStringNodeRelease($2);
                    BDStringNodeRelease($4);
                    BDStringNodeRelease($6);
                }
                
          | key selector VARIABLE
                {
                    $$ = BDQualifierNodeCreateKeyValueNode($1, $2,
                                                           kBDQualifierNodeValueVariable,
                                                           0, NULL, NULL, NULL, $3,
                                                           0);
                    
                    BDQualifierParser_lastParsedNode = $$;

                    BDStringNodeRelease($1);
                    BDStringNodeRelease($2);
                    BDStringNodeRelease($3);
                }

          | key selector TRUE
                {
                    $$ = BDQualifierNodeCreateKeyValueNode($1, $2,
                                                           kBDQualifierNodeValueBoolean,
                                                           0, NULL, NULL, NULL, NULL,
                                                           1);
                    
                    BDQualifierParser_lastParsedNode = $$;
                }

          | key selector FALSE
                {
                    $$ = BDQualifierNodeCreateKeyValueNode($1, $2,
                                                           kBDQualifierNodeValueBoolean,
                                                           0, NULL, NULL, NULL, NULL,
                                                           0);
                    
                    BDQualifierParser_lastParsedNode = $$;
                }


          | key selector key                    /* BDKeyComparisonQualifier */
                {
                    $$ = BDQualifierNodeCreateKeyComparisonNode($1, $2, $3);
                    
                    BDQualifierParser_lastParsedNode = $$;

                    BDStringNodeRelease($1);
                    BDStringNodeRelease($2);
                    BDStringNodeRelease($3);
                }
          | qualifier AND qualifier             /* BDAndQualifier */
                {
                    $$ = BDQualifierNodeCreateAndNode($1, $3);
                    
                    BDQualifierParser_lastParsedNode = $$;
                    
                    BDQualifierNodeRelease($1);
                    BDQualifierNodeRelease($3);
                }
          | qualifier OR qualifier              /* BDOrQualifier */
                {
                    $$ = BDQualifierNodeCreateOrNode($1, $3);
                    
                    BDQualifierParser_lastParsedNode = $$;
                    
                    BDQualifierNodeRelease($1);
                    BDQualifierNodeRelease($3);
                }
          | NOT qualifier                       /* BDNotQualifier */
                {
                    $$ = BDQualifierNodeCreateNotNode($2);
                    
                    BDQualifierParser_lastParsedNode = $$;
                    
                    BDQualifierNodeRelease($2);
                }
          ;

key : IDENTIFIER
                {
                    $$ = $1;
                }
    | KEYPATH
                {
                    $$ = $1;
                }
    ;

selector : EQUAL_TO
                {
                    $$ = BDStringNodeCreate("isEqualTo:");
                }
         | EQUAL_TO_2
                {
                    $$ = BDStringNodeCreate("isEqualTo:");
                }
         | NOT_EQUAL_TO
                {
                    $$ = BDStringNodeCreate("isNotEqualTo:");
                }
         | LESS_THAN
                {
                    $$ = BDStringNodeCreate("isLessThan:");
                }
         | GREATER_THAN
                {
                    $$ = BDStringNodeCreate("isGreaterThan:");
                }
         | LESS_THAN_OR_EQUAL_TO
                {
                    $$ = BDStringNodeCreate("isLessThanOrEqualTo:");
                }
         | GREATER_THAN_OR_EQUAL_TO
                {
                    $$ = BDStringNodeCreate("isGreaterThanOrEqualTo:");
                }
         | CONTAINS
                {
                    $$ = BDStringNodeCreate("doesContain:");
                }
         | LIKE
                {
                    $$ = BDStringNodeCreate("isLike:");
                }
         | CASE_INSENSITIVE_LIKE
                {
                    $$ = BDStringNodeCreate("isCaseInsensitiveLike:");
                }
         ;
      
%%
